(deftemplate transaction
  (slot accountId)
  (slot buyPrice)
  (slot countryCode)
  (slot currentPrice)
  (slot ticker)
  (slot cusip)
  (slot exchange)
  (slot industryGroupID)
  (slot industryID)
  (slot issuer)
  (slot lastPrice)
  (slot purchaseDate)
  (slot sectorID)
  (slot shares)
  (slot subIndustryID)
  (slot total)
  (slot txType)
)
(deftemplate securityHistory
  (slot ticker)
  (slot dailyAverage)
  (slot openPrice)
  (slot closePrice)
  (slot tradeVolume)
  (slot fitchLong)
)
(defcube StockStats
  <- (transaction
    (ticker ?tkr)
    (countryCode ?country)
    (shares ?share)
    (buyPrice ?bprice)
    (total ?total)
  )
  <- (securityHistory
    (ticker ?tkr) ;; join on transaction ticker
    (dailyAverage ?dayavg)
    (openPrice ?oprice)
    (closePrice ?cprice)
    (tradeVolume ?tvol)
    (fitchLong ?flong)
  )
  (dimension ticker ?tkr)
  (dimension countryCode ?country)
  (dimension shares ?share)
  (dimension price ?bprice)
  (dimension total ?total)
  (dimension dailyAverage ?dayavg)
  (dimension openPrice ?oprice)
  (dimension closePrice ?cprice)
  (dimension tradeVolume ?tvol)
  (dimension fitchLong ?flong)
  (aveVolume (average ?tvol))
  (volume (sum ?share))
  (volumeStDev (stdev ?tvol))
  (aveBuy (average ?bprice))
  (maxTotal (max ?total))
  (minTotal (min ?total))
  (maxShares (max ?share))
  (minShares (min ?share))
  (maxBuy (max ?bprice))
  (minBuy (min ?bprice))
  (aveClosing (average ?cprice))
)

(assert (transaction (accountId "ABC")(ticker "MSFT")(countryCode "US")(shares 5000)(buyPrice 50.25)(total 251250) ) )
(assert (transaction (accountId "DEF")(ticker "MSFT")(countryCode "US")(shares 5000)(buyPrice 50.00)(total 250000) ) )
(assert (transaction (accountId "GHI")(ticker "MSFT")(countryCode "US")(shares 5000)(buyPrice 50.20)(total 251000) ) )
(assert (transaction (accountId "JKL")(ticker "MSFT")(countryCode "US")(shares 5000)(buyPrice 50.00)(total 250000) ) )
(assert (securityHistory (ticker "MSFT")(dailyAverage 52.50)(openPrice 48.02)(closePrice 49.80)(fitchLong "AA") ) )
